-- This .love is meant to demonstrate one way of doing a fixed timestep. A fixed timestep forces the
-- game to update only at specific intervals keeping the logic and the rendering seperate. The main
-- benefit of this is to keep the game logic consistant across all platforms of varying speed.

local sAccum = 0 			-- Step accumulator. "Charges" up the time until we have enough for a step.
local sSize = 0.5 			-- The size of a complete step.
local sProgress = 0 		-- The progress to the next step. 0 - 0% progress ; 1 = 100% progress
local lg = love.graphics 	-- We use this a lot so let's make a shortcut


-- Our game object
local object = {
	prev = {},		-- previous state
	x = 0,			-- X location
	y = 0,			-- Y location
	rot = 0,		-- rotation
	scale = 1,		-- scaling
	image = lg.newImage("hamster.png"),	-- The image of our object
}

-- Time step. This happens whe
function object:step()
	-- Take all the numbers in the object and put them into object.prev
	for k,v in pairs(self) do
		if type(v) == "number" then
			self.prev[k] = v
		end
	end
	-- Control the object
	self.x, self.y = love.mouse.getPosition()
	-- Here I based the controls off of the step size. Normally you do not want to do this because
	-- the step size would not change but it does in this .love for demonstration purposes.
	if love.mouse.isDown('r') then self.rot = self.rot + sSize * math.pi end
	if love.mouse.isDown('l') then self.scale = self.scale + sSize end
	if self.scale > 3 then self.scale = 1 end
end
-- Go ahead and call the step so object.prev has numbers
object:step()

-- Get drawing information. This tweens the previous and current dimensions depending
-- on the progress to the next step
function object:getDraw(k)
	return self.prev[k] + (self[k] - self.prev[k]) * sProgress
end

-- Update
function love.update(dt)
	-- If our step size is zero or below then just exit the function
	if sSize <= 0 then return end
	-- Accumulate the time change
	sAccum = sAccum + dt
	-- While we have enough time accumulated fire the step callback.
	while sAccum >= sSize do
		 -- This is where you want to put all of your game updates
		object:step()
		sAccum = sAccum - sSize 
	end
	-- Update the progress to the next step
	sProgress = sAccum / sSize
end

-- Step size controls
function love.mousepressed(x,y,mb)
	if mb == "wu" then sSize = sSize + 0.05 end
	if mb == "wd" then sSize = sSize - 0.05 end
	if sSize < 0.05 then sSize = 0.05 end
	if sSize > 1.00 then sSize = 1.00 end
end

-- Draw
function love.draw()
	-- Draw the object
	lg.draw( object.image, object:getDraw('x'), object:getDraw('y'),
						object:getDraw('rot'), object:getDraw('scale'), object:getDraw('scale'),
						object.image:getWidth()/2, object.image:getHeight()/2)
	-- Print out the information
	lg.print("Mouse wheel: change inteval ; Left Mouse: change size ; Right Mouse: change rotation." ,5, 5)
	lg.print(string.format("Updating every %g seconds.",sSize), 5, 25)
	lg.print(string.format("sProgress: %g", sProgress),5,45)
	lg.print(string.format("previous : (%d,%d) rot: %g  scale: %g", object.prev.x, object.prev.y, object.prev.rot, object.prev.scale),5, 65)
	lg.print(string.format("draw       : (%d,%d) rot: %g  scale: %g", object:getDraw('x'), object:getDraw('y'), object:getDraw('rot'), object:getDraw('scale')),5, 85)
	lg.print(string.format("current    : (%d,%d) rot: %g  scale: %g", object.x, object.y, object.rot, object.scale) ,5, 105)
end